
#include "AGM65G.h"
#include <cmath>
#include <cstdio>

void AGM65LiftFuncVH(double aoa, double M, double Re, double* cl, double *cm, double* cd)
{
	*cl = 3.0 * sin(2*aoa);
	*cm = -0.0012 * sin(aoa);
	*cd = 0.012 + sin(aoa) * 0.112 + 0.0015 * oapiGetInducedDrag(*cl, 0.747, 0.2)+ 0.03*oapiGetWaveDrag(M, 0.95, 1.1, 1.15, 0.1);
}

AGM65G::AGM65G(OBJHANDLE hVessel, int iFlightModel)
	:	uwuwWIDGETCONTAINER(hVessel, iFlightModel), 
		warhead(this, _V(0.0, 0.0, -0.2), AGM65G_WARHEADMASS, 0.4, 0.3)
{
	fBurntime = -1.0;
	hTarget = NULL;
	fLastSimT = 0.0;
	fDistanceTraveled = -1.0;
	fActivated = false;
	fFlightTime = -1.0;
	fArmed = false;
	fAttached = false;
	phTX633 = NULL;
	thTX633B = NULL;
	thTX633 = NULL;
	ahDeployer = NULL;
	iMeshID = 0;
	AnimSeekerA = NULL;
	AnimSeekerE = NULL;
	ElevationAnim = NULL;
	AzimuthAnim = NULL;
	dYaw = 0;

	//----------------------------------------------------
	DefineAnimations();
}

AGM65G::~AGM65G()
{
	delete ElevationAnim;
	delete AzimuthAnim;
}

int AGM65G::ActivateSeeker()
{
	fActivated = true;
	iSeekerMode = -4;
	return 0;
}

bool AGM65G::AquireTargets()
{
	VECTOR3 GPos, LPos;
	double fObjectEl, fObjectAz;

	ClearSeekerList();
	
	int iAquiredObjects = 0;

	int iNumVessels = oapiGetVesselCount();

	for(int i = 0; i<iNumVessels; i++)
	{
		OBJHANDLE hObject = oapiGetVesselByIndex(i);
		if(hObject != GetHandle() )
		{
				
			
			oapiGetGlobalPos(hObject, &GPos);
			double fOSize = oapiGetSize(hObject);
			
			Global2Local(GPos, LPos);		
							
			if(CalculateFOVAngles(LPos, &fObjectAz, &fObjectEl))
			{
				double fDistance = length(LPos);
				double fdAz = fObjectAz - fSeekerAzimuth;
				double fdEl = fObjectEl - fSeekerElevation;
				double fError = sqrt(pow(fdAz, 2) + pow(fdEl, 2));
				//If the object is visible in the seeker.
				if(fError < 0.5 * AGM65G_SEEKERFOV)
				{
					double fRSize = asin(fOSize/fDistance);
					//The object is visible if the seen size is bigger then a halfpixel
					if(fRSize>= sin(AGM65G_SEEKERFOV) / 320) {
						//Insert the object in the seeker object list
						InsertInSeekerList(hObject, fdAz, fdEl, fRSize, fError);
						iAquiredObjects++;
					}
				}
			}
		}
	}
	if(iAquiredObjects > 0)
	{
		return true;
	}
	else
	{
		return false;
	}

}

int AGM65G::Arm()
{
	if(fActivated)
	{
		fArmed = true;
		warhead.Arm();
		return 0;
	}
	else
		return -2;
}

bool AGM65G::CalculateFOVAngles(VECTOR3 &locPos, double *pfAzimuth, double *pfElevation)
{
	double fDist =length(locPos);
	VECTOR3 vZAxis = _V(0.0, 0.0, 1.0);
	double fT0 = CalculatePerpendicularT0(vZAxis, locPos);
	if(fT0<= 0 || fDist == 0.0)
	{
		return false;
	}
	else{
		VECTOR3 P = _V(0.0, 0.0, fT0);
		//The orthogonal error vector 
		VECTOR3 E = locPos - P;
		*pfAzimuth = asin(E.x/fDist);
		*pfElevation = asin(E.y/fDist);
		return true;
	}

}

double AGM65G::CalculatePerpendicularT0(VECTOR3 &Dir, VECTOR3 &x)
{
	
	double lDir = length(Dir);
	double lX = length(x);
	if(lX * lDir == 0.0) {
		return 0.0;
	}
	return dotp(x, Dir)/(lX * lDir);
	
}

VECTOR3 AGM65G::CalculatePointP(VECTOR3 &Dir, VECTOR3 &x)
{

	return Dir * CalculatePerpendicularT0(Dir, x);

}

void AGM65G::ClearSeekerList()
{
	for(int i = 0; i<10; i++)
	{
		hSObject[i] = NULL;
	}
}


void AGM65G::ControlBank()
{
	double bank = GetBank();
	double ebank = -bank;
	if(bank<-0.25 * RAD || bank>0.25 * RAD)
	{
		if(ebank > 10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_AILERON, -1.0);
		} else if(ebank < -10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_AILERON, 1.0);
		}
		else
		{
			SetControlSurfaceLevel(AIRCTRL_AILERON, -ebank/(10 * RAD));
		}
	} 
}

void AGM65G::ControlPitch()
{
	double pitch = GetPitch();
	double epitch = fTPitch-pitch;
	if(epitch<-0.25 * RAD || epitch>0.25 * RAD)
	{
		if(epitch > 10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_ELEVATOR, 1.0);
		} else if(epitch < -10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_ELEVATOR, -1.0);
		}
		else
		{
			SetControlSurfaceLevel(AIRCTRL_ELEVATOR, epitch/(10 * RAD));
		}
	} 

}

void AGM65G::ControlYaw()
{
	double yaw = dYaw;
	double eyaw = -dYaw;
	if(eyaw<-0.25 * RAD || eyaw>0.25 * RAD)
	{
		if(eyaw > 10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_RUDDER, -1.0);
		} else if(eyaw < -10 * RAD)
		{
			SetControlSurfaceLevel(AIRCTRL_RUDDER, 1.0);
		}
		else
		{
			SetControlSurfaceLevel(AIRCTRL_RUDDER, -eyaw/(10 * RAD));
		}
	} 

}

void AGM65G::DefineAnimations()
{
	unsigned int GroupSeeker[1] = {1};
	unsigned int GroupGimbal[1] = {2};

	//RegisterAnimation();

	//ElevationAnim = new MGROUP_ROTATE(0, GroupGimbal, 1, _V(0.0, 0.0, 1.1), _V(1.0, 0.0, 0.0),
		//(float)(2 * 27.5 * RAD));
	//AzimuthAnim = new MGROUP_ROTATE(0, GroupSeeker, 1, _V(0.0, 0.0, 1.1), _V(0.0, 1.0, 0.0),
		//(float)(2 * 27.5 * RAD));

	//AnimSeekerE = CreateAnimation(0.5);
	//AnimSeekerA = CreateAnimation(0.5);
	//ANIMATIONCOMPONENT_HANDLE hParentAnim = AddAnimationComponent(AnimSeekerA, 0.0, 1.0, 
	//	AzimuthAnim);

	//AddAnimationComponent(
	//	AnimSeekerE, 0.0, 1.0, ElevationAnim);
	
	RegisterAnimation();
	
	
}

double AGM65G::GetNeutralAOA()
{
	double W = GetMass()/AGM65G_WINGAREA;
	double cl = W * 9.81 / GetDynPressure();
	return asin(cl/3.0);
}

bool AGM65G::InSeekerFOV(VECTOR3 &vFOVRel, VECTOR3 &RelPos)
{
	const double SEEKERFOV = 5.0 * RAD;
	const double MAXSEEKERRANGE = 35000.0;
	double fDistance = length(RelPos);
			
	if(fDistance < MAXSEEKERRANGE && fDistance > 0.00)	//maximum seeker Range
	{
		double fT0 = CalculatePerpendicularT0(vFOVRel, RelPos); 
		if(fT0> 0)
		{	
			
			VECTOR3 P = vFOVRel * fT0;
			double A = dist(P, RelPos);
			
			if(acos(A/fT0) <= 0.5 * SEEKERFOV)
			{
				return true;
			} else
			{ 
				return false;
			}
		}
		else
		{
			return false;
		}
		
	}
	else
	{
		return false;
	}
}

void AGM65G::InsertInSeekerList(OBJHANDLE hObject, double fdAz, double fdEl, double fRSize, double fError)
{
	int i = 0;
	while(hSObject[i] != NULL && i<10)
	{
		if(fSObjectError[i] > fError || 
				(fSObjectError[i] == fError && fRSize > fSObjectRSize[i]))
		{
			//Insert here.
			for(int j = 9; j> i+1; j--)
			{
				hSObject[j] = hSObject[j-1];
				fSObjectError[j] = fSObjectError[j-1];
				fSObjectRAz[j] = fSObjectRAz[j-1];
				fSObjectREl[j] = fSObjectREl[j-1];
				fSObjectRSize[j] = fSObjectRSize[j-1];
			}
			hSObject[i] = hObject;
			fSObjectError[i] = fError;
			fSObjectRAz[i] = fdAz;
			fSObjectREl[i] = fdEl;
			fSObjectRSize[i] = fRSize;
			return;
		}
		i++;
	}
	if(i<10)
	{
		hSObject[i] = hObject;
		fSObjectError[i] = fError;
		fSObjectRAz[i] = fdAz;
		fSObjectREl[i] = fdEl;
		fSObjectRSize[i] = fRSize;
	}
}

void AGM65G::MoveSeeker(double fSimT, double fDeltaT)
{
	
	
	const double SEEKERFOV = 5.0 * RAD;
	const double MAXSEEKERRANGE = 35000.0;

	const double AZ_MIN = -27.5 * RAD;
	const double EL_MIN = -27.5 * RAD;
	const double AZ_MAX = 27.5 * RAD;
	const double EL_MAX = 27.5 * RAD;
	const double SEEKER_AZIMUTH_SP1[12] = 
	{  0.00		 , 13.75 * RAD, 23.82 * RAD, 
	  27.50 * RAD, 28.82 * RAD, 13.75 * RAD,
	   0.00		 ,-13.75 * RAD,-23.82 * RAD,
	   -27.50 * RAD,-23.82 * RAD,-13.75 *RAD};
	
	const double SEEKER_ELEVATION_SP1[12] = 
	{ 27.50 * RAD, 23.82 * RAD, 13.75 * RAD, 
	   0.00      ,-13.75 * RAD, -23.82 * RAD,
	 -27.50 * RAD,-23.82 * RAD,-13.75 * RAD,
	   0.00		 , 13.75 * RAD, 23.82 *RAD};
	
	double fSP1_Az = SEEKER_AZIMUTH_SP1[iSeekerStep];
	double fSP1_El = SEEKER_ELEVATION_SP1[iSeekerStep];
	double fSP2_Az = -fSP1_Az;
	double fSP2_El = -fSP1_El;
	
	fTargetAquired = false;

	double fdAz = 0;
	double fdEl = 0;

	double fError = 0;

	double fVAz = 0;
	double fVEl = 0;

	bool fTargetsVisible = AquireTargets();

	
	switch(iSeekerMode)
	{
	case -1:
		//Disabled
		break;
	case -2:
		//Bore
		MoveSeekerTo(0.0, 0.0, fDeltaT);
		break;
	case -3:
		//Follow
		if(fTargetsVisible)
		{
			//Take the closest object (Index 0)
			MoveSeekerTo(fSeekerAzimuth + fSObjectRAz[0], 
				fSeekerElevation + fSObjectREl[0], 
				fDeltaT);
			fTargetAquired = true;
		}
		else {
			iSeekerMode = -4;
		}
		break;
	case -4:
		//move to start Position (0,0)

		MoveSeekerTo(0.0, 0.0, fDeltaT);

		if(fSeekerAzimuth == 0.0 && fSeekerElevation == 0.0)
		{
			iSeekerMode = 0;
			iSeekerStep = 0;
		}
		break;
	case 0:
		
		//Move Seeker up to Position 1
		MoveSeekerTo(fSP1_Az, fSP1_El, fDeltaT);
		
		if(fSeekerElevation == fSP1_El && fSeekerAzimuth == fSP1_Az)
		{
			iSeekerMode = 1;
		}
		
		break;
	case 1:
		//Move down to position 2
		MoveSeekerTo(fSP2_Az, fSP2_El, fDeltaT);

		if(fSeekerElevation == fSP2_El && fSeekerAzimuth == fSP2_Az)
		{
			iSeekerMode = 2;
		}
		break;
	case 2:
		//Center again
		MoveSeekerTo(0.0, 0.0, fDeltaT);
		if(fSeekerAzimuth == 0.0 && fSeekerElevation == 0.0)
		{
			iSeekerStep += 5;
			if(iSeekerStep >= 12)
				iSeekerStep = iSeekerStep - 12;
			iSeekerMode = 0;
		}
		
		break;
	}

	if(fTargetsVisible && iSeekerMode >= 0)
	{
		iSeekerMode = -3;
		fTargetAquired = true;
	}
	
}

void AGM65G::MoveSeekerTo(double fTAz, double fTEl, double fDeltaT)
{
		double fRotSpeed = AGM65G_SEEKERSPEED * fDeltaT;
		double fdAz = fTAz - fSeekerAzimuth;
		double fdEl = fTEl - fSeekerElevation;
	
		double fError = sqrt(pow(fdAz, 2) + pow(fdEl, 2));
		if(fError == 0.0)
		{
			return;
		}
	
		double fVAz = fdAz/fError * fRotSpeed;
		double fVEl = fdEl/fError * fRotSpeed;

		if(fVEl <0)
		{
			if(fSeekerElevation + fVEl > fTEl)
			{
				fSeekerElevation += fVEl;
			} else
			{
				fSeekerElevation = fTEl;
			}
		} else
		{
			if(fSeekerElevation + fVEl < fTEl)
			{
				fSeekerElevation += fVEl;
			} else
			{
				fSeekerElevation = fTEl;
			}
		}

		if(fVAz <0)
		{
			if(fSeekerAzimuth + fVAz > fTAz)
			{
				fSeekerAzimuth += fVAz;
			} else
			{
				fSeekerAzimuth = fTAz;
			}
		} else
		{
			if(fSeekerAzimuth + fVAz < fTAz)
			{
				fSeekerAzimuth += fVAz;
			} else
			{
				fSeekerAzimuth = fTAz;
			}
		}

}

void AGM65G::clbkAnimate(double fSimT)
{
	cDebugAnimated = 'A';
	double fAngle = 0.0;
	if(fSkrAnimE != fSeekerElevation)
	{
		fAngle = fSeekerElevation - fSkrAnimE;
		MESHGROUP_TRANSFORM mt;
		mt.transform = MESHGROUP_TRANSFORM::ROTATE;
		mt.ngrp = 25;
		mt.nmesh = 0;
		mt.P.rotparam.ref = _V(0.0, 0.0, 1.1);
		mt.P.rotparam.axis = _V(1.0, 0.0, 0.0);
		mt.P.rotparam.angle = fAngle;
		MeshgroupTransform(MyVis, mt);
		mt.ngrp = 24;
		MeshgroupTransform(MyVis, mt);
		//
		fSkrAnimE = fSeekerElevation;
	}
	if(fSkrAnimA != fSeekerAzimuth)
	{
		fAngle = fSeekerAzimuth - fSkrAnimA ;
		MESHGROUP_TRANSFORM mt;
		mt.ngrp = 24;
		mt.transform = MESHGROUP_TRANSFORM::ROTATE;
		mt.nmesh = 0;
		mt.P.rotparam.ref = _V(0.0, 0.0, 1.1);
		mt.P.rotparam.axis = _V(0.0, cos(fSeekerElevation), 
			sin(fSeekerElevation));
		mt.P.rotparam.angle = fAngle;
		MeshgroupTransform(MyVis, mt);
		//
		fSkrAnimA = fSeekerAzimuth;
	}

}

int AGM65G::clbkConsumeBufferedKey(DWORD dwKey, bool fPressed, char* kstate)
{
	if(KEYMOD_SHIFT(kstate))
		return 0;
	if(KEYMOD_CONTROL(kstate))
	{
		return 0;
	}
	else
	{
		if(dwKey == OAPI_KEY_I && !fPressed)
		{
			Arm();
			fBurntime = 0.0;
			return 1;
		} 
		else if (dwKey == OAPI_KEY_O && !fPressed)
		{
			if(!fActivated)
				ActivateSeeker();
			return 1;
		}
		return 0;
	}
}


void AGM65G::onSetClassCaps(FILEHANDLE cfg)
{
	SURFHANDLE exhaust2 = oapiRegisterExhaustTexture("exhaust2");
	static PARTICLESTREAMSPEC tx633_contrail = {
		0, 0.3, 5, 150.0, 0.3, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
	};
	static PARTICLESTREAMSPEC tx633_exhaust = {
		0, 0.1, 20, 150.0, 0.1, 0.3, 12, 2.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
	};
	

	SetSize(1.25);
	SetEnableFocus(false);
	SetEmptyMass(AGM65G_EMPTYMASS);
	SetPMI(_V(0.38, 0.38, 0.02));
	SetCrossSections(_V(1.02, 1.02, 0.09));
	iMeshID = AddMesh(oapiLoadMeshGlobal("AGM65G"));
	
	SetTouchdownPoints(_V(0.0, -0.38, 1.0), _V(-1.0, -0.35, -1.0), 
		_V(1.0, -0.35, -1.0));

	ahDeployer = CreateAttachment(true, _V(0.0, 0.2, 0.0), _V(0.0, 1.0, 0.0), _V(0.0, 0.0, 1.0), "WM");
	
	//---------------------------------------------
	if(phTX633 == NULL)
	{
		phTX633 = CreatePropellantResource(TX633_PROPELLANTMASS);
	}
	//Create rocket engine booster thruster to simulate the extra boost for the start.
	if(thTX633B == NULL)
	{
		thTX633B = CreateThruster(_V(0,0,-1.0), _V(0,0,1), (TX633_BOOST_THRUST - TX633_SUSTAINER_THRUST), phTX633, 
			TX633_ISP, TX633_ISP);
	}
	if(thTX633 == NULL)
	{
		thTX633 = CreateThruster(_V(0,0,-1.0), _V(0,0,1), TX633_SUSTAINER_THRUST, phTX633, 
			TX633_ISP, TX633_ISP);
	}
	

	CreateAirfoil(LIFT_VERTICAL, _V(0.0, 0.0, -0.09), &AGM65LiftFuncVH, 
		1.4, AGM65G_WINGAREA, 0.747);
	CreateAirfoil(LIFT_HORIZONTAL, _V(0.0, 0.0, -0.09), &AGM65LiftFuncVH, 
		1.4, AGM65G_WINGAREA, 0.747);
	CreateControlSurface(AIRCTRL_AILERON, 0.07, +0.35, _V(-0.25, 0.0, -1.1));
	CreateControlSurface(AIRCTRL_AILERON, 0.07, +0.35, _V( 0.25, 0.0, -1.1));
	CreateControlSurface(AIRCTRL_ELEVATOR, 0.14, +0.35, _V(0.0, 0.0, -1.1));
	CreateControlSurface(AIRCTRL_RUDDER, 0.14, +0.35, _V(0.0, 0.0, -1.1));
	AddExhaust(thTX633, 1.5, 0.15, 0.35, exhaust2);
	AddExhaust(thTX633B, 3.5, 0.15, 0.35, exhaust2);
	AddExhaustStream(thTX633B, _V(0.0, 0.0, -1.30), &tx633_exhaust);
	AddExhaustStream(thTX633B, _V(0.0, 0.0, -1.50), &tx633_contrail);
	//-----------------------------------------------------------------------
	
}



void AGM65G::clbkLoadStateEx(FILEHANDLE scn, void* vs)
{
	char* line;
	while(oapiReadScenario_nextline(scn, line))
	{
		if(!strnicmp(line, "BURNTIME", 8))
		{
			sscanf(line+8, "%lf", &fBurntime);
		}
		else
			ParseScenarioLineEx(line, vs);
	}
}

void AGM65G::clbkSaveState(FILEHANDLE scn)
{
	oapiWriteScenario_float(scn, "BURNTIME", fBurntime);
}

void AGM65G::onTimestep(double fSimT, double fDeltaT)
{
	double Dummy, Long, Lat;

	GetEquPos(Long, Lat, Dummy);
	if(fAttached)
	{
		if(GetAttachmentStatus(ahDeployer) == NULL)
		{
			fAttached = false;
			if(fArmed)
			{
				fFlightTime = 0.0;
				fBurntime = 0.0;
			}
		}
	} else
	{
		if(GetAttachmentStatus(ahDeployer) != NULL)
		{
			fAttached = true;
			if(fBurntime >= 0.0)
			{
				fBurntime = -1.0;
				fFlightTime = -1.0;
				SetPropellantMass(phTX633, TX633_PROPELLANTMASS);
			}
		}
	}

	if(GetAltitude() > 5.0 && !fActivated) {
		//created in flight
		ActivateSeeker();
		Arm();
		fAttached = false;
		fTPitch = GetPitch();
		fBurntime = 0.0;
		fFlightTime = 0.0;
	}

	if(fActivated)
	{
		MoveSeeker(fSimT, fDeltaT);
	}

	if(fFlightTime >= 0.0)
	{
		ControlBank();
		ControlPitch();
		ControlYaw();
		
		fFlightTime += fDeltaT;
		if(fFlightTime > 3.0) 
		{
			//Start aiming for target.
			if(fTargetAquired) {
				fTPitch = GetPitch() + fSeekerElevation + GetNeutralAOA();
				dYaw = fSeekerAzimuth;
			} else 
			{
				//Fly straight and level
				fTPitch = GetNeutralAOA();
				dYaw = 0.0;
			}
		}
		if(fFlightTime > AGM65G_RSTIME) 
		{
			//Ignite Warhead for range safety.
			warhead.Ignite();
			sprintf(oapiDebugString(), "%s: Exceeded flight time limit of %5.1f seconds", 
				GetName(), AGM65G_RSTIME);
		}
	}
	
	if(fBurntime >= 0 && GetPropellantMass(phTX633)>0.0)
	{
		
		fBurntime += fDeltaT;
		SetThrusterLevel(thTX633, 1.0);
		if(fBurntime <= TX633_BOOSTPHASE_DURATION)
		{
			SetThrusterLevel(thTX633B, 1.0);
		} else
		{
			SetThrusterLevel(thTX633B, 0.0);
		}
		
	
		if(fDistanceTraveled < 0)
		{
			fLastLong = Long;
			fLastLat = Lat;
			fDistanceTraveled = 0;
		} 
		
	}

	if(fDistanceTraveled >= 0.0)
	{
		fDistanceTraveled += sqrt(pow(sin(Long - fLastLong) * 6571, 2) + pow(sin(Lat - fLastLat) * 6571, 2));
		fLastLong = Long;
		fLastLat = Lat;
	}
	/*
	sprintf(oapiDebugString(), "Seeker: %+2d [%4.1f, %4.1f] %c [%3.1f, %3.1f] %8X=%s", iSeekerMode, 
		fSeekerAzimuth * DEG, fSeekerElevation * DEG, cDebugAnimated, 
		fSkrAnimA * DEG, fSkrAnimE * DEG,
		hTarget, pszTargetName);
	*/
	cDebugAnimated = 'N';
	
}

void AGM65G::clbkVisualCreated(VISHANDLE vis, int iRefCount)
{
	MyVis = vis;
}

void AGM65G::clbkVisualDestroyed(VISHANDLE vis, int iRefCount)
{
}


int AGM65G::Safe()
{
	if(fAttached)
	{
		fArmed = false;
		return 0;
	}
	else
		return -1;
}

int AGM65G::SendMessage(const char* pszMessage, void* aMessage)
{
	if(!strnicmp(pszMessage, "WEAPON::", 8))
	{
		if(!strnicmp(pszMessage+8, "GETTYPE", 7))
		{
			return 0x01;
		}
		else if(!strnicmp(pszMessage+8, "ARM", 3))
		{
			return Arm();
			
		}
		else if(!strnicmp(pszMessage+8, "SAFE", 4))
		{
			return Safe();
		}
		else if(!strnicmp(pszMessage+8, "ACTIVATE", 8))
		{
			return ActivateSeeker();
		}
		else
			return -1;
	}
	else
		return 0;
}

